@@ -12,12 +12,13 @@ You may obtain a copy of the License at
    See the License for the specific language governing permissions and
    limitations under the License."""
 
-from django.conf.urls.defaults import *
+from django.conf.urls import url
+from . import views
 
-urlpatterns = patterns('graphite.browser.views',
-  ('^header/?$', 'header'),
-  ('^search/?$', 'search'),
-  ('^mygraph/?$', 'myGraphLookup'),
-  ('^usergraph/?$', 'userGraphLookup'),
-  ('', 'browser'),
-)
+urlpatterns = [
+    url(r'^/header/?$', views.header, name='browser_header'),
+    url(r'^/search/?$', views.search, name='browser_search'),
+    url(r'^/mygraph/?$', views.myGraphLookup, name='browser_my_graph'),
+    url(r'^/usergraph/?$', views.userGraphLookup, name='browser_usergraph'),
+    url(r'^/?$', views.browser, name='browser'),
+]
