@@ -26,6 +26,17 @@ from graphite.storage import STORE
 from graphite.util import timebounds, logtime
 
 
+try:
+  from collections import UserDict
+except ImportError:
+  from UserDict import IterableUserDict as UserDict
+
+
+class Tags(UserDict):
+  def __setitem__(self, key, value):
+    self.data[key] = str(value)
+
+
 class TimeSeries(list):
   def __init__(self, name, start, end, step, values, consolidate='average', tags=None, xFilesFactor=None, pathExpression=None):
     list.__init__(self, values)
@@ -76,36 +87,47 @@ class TimeSeries(list):
   __consolidation_functions = {
     'sum': sum,
     'average': lambda usable: sum(usable) / len(usable),
+    'avg_zero': lambda usable: sum(usable) / len(usable),
     'max': max,
     'min': min,
     'first': lambda usable: usable[0],
     'last': lambda usable: usable[-1],
   }
+  __consolidation_function_aliases = {
+    'avg': 'average',
+  }
 
   def __consolidatingGenerator(self, gen):
-    try:
+    if self.consolidationFunc in self.__consolidation_functions:
       cf = self.__consolidation_functions[self.consolidationFunc]
-    except KeyError:
+    elif self.consolidationFunc in self.__consolidation_function_aliases:
+      cf = self.__consolidation_functions[self.__consolidation_function_aliases[self.consolidationFunc]]
+    else:
       raise Exception("Invalid consolidation function: '%s'" % self.consolidationFunc)
 
-    buf = []  # only the not-None values
+    buf = []
     valcnt = 0
+    nonNull = 0
 
     for x in gen:
       valcnt += 1
       if x is not None:
         buf.append(x)
+        nonNull += 1
+      elif self.consolidationFunc == 'avg_zero':
+        buf.append(0)
 
       if valcnt == self.valuesPerPoint:
-        if buf and (len(buf) / self.valuesPerPoint) >= self.xFilesFactor:
+        if nonNull and (nonNull / self.valuesPerPoint) >= self.xFilesFactor:
           yield cf(buf)
         else:
           yield None
         buf = []
         valcnt = 0
+        nonNull = 0
 
     if valcnt > 0:
-      if buf and (len(buf) / self.valuesPerPoint) >= self.xFilesFactor:
+      if nonNull and (nonNull / self.valuesPerPoint) >= self.xFilesFactor:
         yield cf(buf)
       else:
         yield None
@@ -147,6 +169,19 @@ class TimeSeries(list):
     timestamps = range(int(self.start), int(self.end) + 1, int(self.step * self.valuesPerPoint))
     return list(zip(self, timestamps))
 
+  @property
+  def tags(self):
+    return self.__tags
+
+  @tags.setter
+  def tags(self, tags):
+    if isinstance(tags, Tags):
+      self.__tags = tags
+    elif isinstance(tags, dict):
+      self.__tags = Tags(tags)
+    else:
+      raise Exception('Invalid tags specified')
+
 
 # Data retrieval API
 @logtime
@@ -276,6 +311,10 @@ def prefetchData(requestContext, pathExpressions):
   if not requestContext.get('prefetched'):
     requestContext['prefetched'] = {}
 
-  requestContext['prefetched'][(startTime, endTime, now)] = prefetched
+  if (startTime, endTime, now) in requestContext['prefetched']:
+    for k in prefetched:
+      requestContext['prefetched'][(startTime, endTime, now)][k] = prefetched[k]
+  else:
+      requestContext['prefetched'][(startTime, endTime, now)] = prefetched
 
   log.rendering("Fetched data for [%s] in %fs" % (', '.join(pathExpressions), time.time() - start))
