@@ -12,8 +12,11 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License."""
 
-import math, itertools, re
+import itertools
+import math
+import re
 from datetime import datetime, timedelta
+
 from six.moves import range, zip
 from six.moves.urllib.parse import unquote_plus
 from six.moves.configparser import SafeConfigParser
@@ -87,7 +90,7 @@ try:
         percent_l_supported = True
     else:
         percent_l_supported = False
-except ValueError as e:
+except ValueError:
     percent_l_supported = False
 
 DATE_FORMAT = settings.DATE_FORMAT
@@ -517,8 +520,8 @@ class _LogAxisTics(_AxisTics):
 
 
 class Graph:
-  customizable = ('width','height','margin','bgcolor','fgcolor', \
-                 'fontName','fontSize','fontBold','fontItalic', \
+  customizable = ('width','height','margin','bgcolor','fgcolor',
+                 'fontName','fontSize','fontBold','fontItalic',
                  'colorList','template','yAxisSide','outputFormat')
 
   def __init__(self,**params):
@@ -793,6 +796,12 @@ class Graph:
           y += lineHeight
 
   def encodeHeader(self,text):
+    """
+    Puts some metadata in the generated svg xml that is not inside the frame.
+    This can be used to manipulate the svg later on with a framework like d3.
+    """
+    if self.outputFormat != 'svg':
+      return
     self.ctx.save()
     self.setColor( self.backgroundColor )
     self.ctx.move_to(-88,-88) # identifier
@@ -882,10 +891,10 @@ class Graph:
         metaData = { }
 
       self.surface.finish()
-      svgData = str(self.surfaceData.getvalue())
+      svgData = self.surfaceData.getvalue().decode('utf-8')
       self.surfaceData.close()
 
-      svgData = svgData.replace('pt"', 'px"', 2) # we expect height/width in pixels, not points
+      svgData = svgData.replace('pt"', 'px"', 2)  # we expect height/width in pixels, not points
       svgData = svgData.replace('</svg>\n', '', 1)
       svgData = svgData.replace('</defs>\n<g', '</defs>\n<g class="graphite"', 1)
 
@@ -900,10 +909,10 @@ class Graph:
         (svgData, subsMade) = re.subn(r'<path.+?d="M -88 -88 (.+?)"/>', onHeaderPath, svgData)
 
         # Replace the first </g><g> with <g>, and close out the last </g> at the end
-        svgData = svgData.replace('</g><g data-header','<g data-header',1)
+        svgData = svgData.replace('</g><g data-header', '<g data-header', 1)
         if subsMade > 0:
-          svgData += "</g>"
-        svgData = svgData.replace(' data-header="true"','')
+          svgData += '</g>'
+        svgData = svgData.replace(' data-header="true"', '')
 
       fileObj.write(svgData.encode('utf-8'))
       fileObj.write(("""<script>
@@ -916,16 +925,16 @@ class Graph:
 
 class LineGraph(Graph):
   customizable = Graph.customizable + \
-                 ('title','vtitle','lineMode','lineWidth','hideLegend', \
-                  'hideAxes','minXStep','hideGrid','majorGridLineColor', \
-                  'minorGridLineColor','thickness','min','max', \
-                  'graphOnly','yMin','yMax','yLimit','yStep','areaMode', \
-                  'areaAlpha','drawNullAsZero','tz', 'yAxisSide','pieMode', \
-                  'yUnitSystem', 'logBase','yMinLeft','yMinRight','yMaxLeft', \
-                  'yMaxRight', 'yLimitLeft', 'yLimitRight', 'yStepLeft', \
-                  'yStepRight', 'rightWidth', 'rightColor', 'rightDashed', \
-                  'leftWidth', 'leftColor', 'leftDashed', 'xFormat', 'minorY', \
-                  'hideYAxis', 'uniqueLegend', 'vtitleRight', 'yDivisors', \
+                 ('title','vtitle','lineMode','lineWidth','hideLegend',
+                  'hideAxes','minXStep','hideGrid','majorGridLineColor',
+                  'minorGridLineColor','thickness','min','max',
+                  'graphOnly','yMin','yMax','yLimit','yStep','areaMode',
+                  'areaAlpha','drawNullAsZero','tz', 'yAxisSide','pieMode',
+                  'yUnitSystem', 'logBase','yMinLeft','yMinRight','yMaxLeft',
+                  'yMaxRight', 'yLimitLeft', 'yLimitRight', 'yStepLeft',
+                  'yStepRight', 'rightWidth', 'rightColor', 'rightDashed',
+                  'leftWidth', 'leftColor', 'leftDashed', 'xFormat', 'minorY',
+                  'hideYAxis', 'uniqueLegend', 'vtitleRight', 'yDivisors',
                   'connectedLimit', 'hideXAxis', 'hideNullFromLegend')
   validLineModes = ('staircase','slope','connected')
   validAreaModes = ('none','first','all','stacked')
@@ -1813,7 +1822,7 @@ class PieGraph(Graph):
     self.valueLabelsMin = float( params.get('valueLabelsMin',5) )
     self.valueLabels = params.get('valueLabels','percent')
     assert self.valueLabels in self.validValueLabels, \
-    "valueLabels=%s must be one of %s" % (self.valueLabels,self.validValueLabels)
+        "valueLabels=%s must be one of %s" % (self.valueLabels,self.validValueLabels)
     if self.valueLabels != 'none':
       self.drawLabels()
 
